;----------------------------------------------------------------------------
;
; Min2Tray
;
; _v1.4, 2006/03/15
;
; _minimize windows to tray-bar as individual icons
;  by pressing third mouse button over titlebar of window
;
; _program is a 2-in-1 thingy:
;   .started without parameters on cmdline it behaves as
;    the main program waiting for hotkey action
;   .startet with ahk_window_id on cmdline by main program
;    it will hide that particular window and generate
;    a (hopefully) meaningful tray icon
;
; _licensed under the terms of the GPLv2 or later 
;
; _created by Junyx / KTC^brain in June 2005
;
; _inspired by: ActualWindowMinimizer (unfortunately payware)
;               minimize.ahk (from AHK docs, no author)
;               NiftyWindows by Enovatic-Solutions (nice tweaks)
;               ac'tivAid by Heise Zeitschriften Verlag GmbH & Co. KG
;
; _thanks to: Chris Mallett (support response in a flash!)
;             Rajat (for some wonderful scripts)
;             GSX-R600 / KTC^game (for testing)
;
;----------------------------------------------------------------------------

;----------------------------------------------------------------------------
; init
;

versionString = 1.4
versionDate   = 20060315  ; YYYYMMDD

Gosub, h_InitGeneral ; l10n etc.

#Persistent
#NoTrayIcon
#UseHook On
#SingleInstance Off
#WinActivateForce
DetectHiddenWindows, On
SetWinDelay 10

If ( %0% < 1 )	; cmdline: no args -> behaviour: starter
{
   ;=========================================================================
   ; started as hotkey-launcher (starter)
	Goto, h_InitStarter
	Return
   ; end of auto-execution for starter here! - redundant, but alas...
   ;=========================================================================
}

;----------------------------------------------------------------------------
; Helper -- init
;

If %1% is not xdigit
	Goto, h_ExitWithParamError

; do not interfere with user while send'ing
BlockInput, Send
; get first cmdline parameter
h_ID = %1%

WinWait, ahk_id %h_ID%,, 2		; set "last used window"
If ( ErrorLevel )			; It timed out, so do nothing
	ExitApp, 1

WinGetClass, h_Class
If h_Class in Shell_TrayWnd,Progman
	Goto, h_ExitWithClassError

WinGetActiveTitle, h_Title
WinGet, h_Application, ProcessName

Send, !{ESC}  ; activate last used window
Sleep, 50
WinHide

OnExit, h_Unhide

Gosub, h_SetMenu
Gosub, h_SetTrayIcon
Gosub, h_SetTimedCheck
; assign user-hotkey, if any
h_hkKey =
RegRead, h_hkKey, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|ch
f_AssignHotkey( h_hkKey, "h_Unhide" )

Return	; end of auto execution, wait for menu action

;----------------------------------------------------------------------------
; Helper -- subs
;

h_Unhide:
 WinShow, ahk_id %h_ID%
 WinActivate, ahk_id %h_ID%
ExitApp

h_Close:
 Msgbox, 36, %lng_WindowTitle%, %lng_hClose1%%h_Title% [%h_Application%]%lng_hClose2%
 IfMsgBox, No	; do not close
	Return
 OnExit
 WinShow, ahk_id %h_ID%
 WinClose, ahk_id %h_ID%
ExitApp

h_ChangeName:
 InputBox, h_appName, %lng_WindowTitle%, %lng_ChangeNameSelector%, , , 125, , , , , %h_MenuWin%
 If ( Not ErrorLevel )
 {
	StringLen, tmp, h_appName
	If ( tmp < 1 )
		Return

	RegWrite, REG_SZ, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|cn, %h_appName%
	Gosub, h_SetMenu
 }
Return

h_RemoveName:
 RegRead, tmp, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|cn	; does custom name exist?
 If ( Not ErrorLevel )
 {
	Msgbox, 36, %lng_WindowTitle%, %lng_RemoveName%
	IfMsgBox, No	; do not remove
		Return
	RegDelete, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|cn
	If ( Not ErrorLevel )
		Gosub, h_SetMenu
 }
Return

h_ChangeIcon:
	; BUG: if user cancels dialog ahk sets the wrong iconfile
	FileSelectFile, h_appExe, 3, , %lng_ChangeIconSelector%, Icon Files (*.ico; *.dll; *.exe)
	IfExist, %h_appExe%
	{
		f_ShowTrayIcon( h_appExe, 1 )
		; write user-changed icon to registry
		; TODO -- dialog for selecting other than icon no. 1
		RegWrite, REG_SZ, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|ci, %h_appExe%|1
	}
Return

h_RemoveIcon:
 RegRead, tmp, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|ci	; does custom icon exist?
 If ( Not ErrorLevel )
 {
	Msgbox, 36, %lng_WindowTitle%, %lng_RemoveIcon%
	IfMsgBox, No	; do not remove
		Return
	RegDelete, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|ci
	If ( Not ErrorLevel )
		Gosub, h_SetTrayIcon
 }
Return

h_SetTimedCheck:
   ; get interval from registry, if any
	RegRead, tmp, HKCU, %h_RegSubkey%\Misc, CheckForWinEveryMiliSec
	If ( ErrorLevel OR tmp < 200 OR tmp > 1000000 )
		tmp = 2500
	SetTimer, h_CheckForWin, %tmp%
Return

h_CheckForWin:
 WinGet, tmp, Style, ahk_id %h_ID%  ; window already unhidden or non existing?
 Transform, tmp, BitAnd, %tmp%, 0xF0000000
 If ( tmp = 0x10000000 OR Not WinExist( "ahk_id " . h_ID ) )
 {
	SetTimer, h_CheckForWin, Off
	Goto, h_Unhide			; if not -> exit this helper-instance
 }
 
 WinGetTitle, tmp, ahk_id %h_ID%	; otherwise update window title and redraw menu and tray
 If ( tmp <> h_Title )
 {
	h_Title = %tmp%
	Gosub, h_SetMenu
 }

	f_ShowTrayIcon( h_appExe, h_appExeIcon )
Return

h_SetMenu:
	RegRead, h_appName, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|cn 	; look for custom name
	If ( Not ErrorLevel )
	{
		h_MenuWin    = %h_appName%
		h_TrayTitle  = %h_appName%
	}
	Else
	{
		RegRead, h_appName, HKCU, %h_RegSubkey%, %h_Application%|cn 	; look for generic custom name
		If ( Not ErrorLevel )
		{
			h_MenuWin    = %h_appName%
			h_TrayTitle  = %h_appName%
		}
		Else
		{
			h_MenuWin    = %h_Title% [%h_Application%]
			h_TrayTitle  = %h_Title%`n[%h_Application%]
		}
	}

	; is this window on BossKey list?
	h_bkEnabled := FALSE	; default: nope
	RegRead, h_bkEnabled, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|bk	; app+class on bosskey list?

	Menu, TRAY, DeleteAll	; tabularasa
	Menu, TRAY, Click, 1	; 1-click menu
	Menu, TRAY, NoStandard
	Menu, TRAY, Add, %h_MenuWin%, h_Unhide
	Menu, TRAY, Add
	Menu, TRAY, Add, %lng_MenuChangeName%, h_ChangeName
	Menu, TRAY, Add, %lng_MenuRemoveName%, h_RemoveName
	Menu, TRAY, Add, %lng_MenuChangeIcon%, h_ChangeIcon
	Menu, TRAY, Add, %lng_MenuRemoveIcon%, h_RemoveIcon
	Menu, TRAY, Add
	If ( h_bkEnabled )
		Menu, TRAY, Add, %lng_MenuOnBKList%, h_BossKeyListToggle
	Else
		Menu, TRAY, Add, %lng_MenuNotOnBKList%, h_BossKeyListToggle
	Menu, TRAY, Add, %lng_MenuAssignHotkey%, h_SetHotkeyHelper
	Menu, TRAY, Add
	Menu, TRAY, Add, %lng_MenuClose%, h_Close
	Menu, TRAY, Add, %lng_MenuUnhide%, h_Unhide
	Menu, TRAY, Default, %lng_MenuUnhide%
	Menu, TRAY, Tip, %h_TrayTitle%
Return

h_SetTrayIcon:
	RegRead, tmp, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|ci	; first look for user icon
	If ( Not ErrorLevel )
	{
		StringSplit, tmparray, tmp, |
		h_appExe     = %tmparray1%
		h_appExeIcon = %tmparray2%
	}
	Else
	{
		RegRead, tmp, HKCU, %h_RegSubkey%, %h_Application%|ci	; look for generic user icon (for app)
		If ( Not ErrorLevel )
		{
			StringSplit, tmparray, tmp, |
			h_appExe     = %tmparray1%
			h_appExeIcon = %tmparray2%
		}
		Else If ( ( h_Application = "explorer.exe" ) AND ( h_Class = "ExploreWClass" ) )
		{	; special treatment for explorer windows
			h_appExe     = %A_WinDir%\system32\shell32.dll
			h_appExeIcon = 5
		}
		Else
		{	; use icon from registry-entry as default, if existing
			h_appExeIcon = 1
			RegRead, h_appExe, HKLM, SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\%h_Application%
			If ( h_appExe = "" )	; otherwise do a file search
			{
				h_appExe := f_FindFile( h_Application, A_WinDir )
				If ( h_appExe = "" )	; nothing found in windir...
					h_appExe := f_FindFile( h_Application, A_ProgramFiles )
			}
		}
	}
	f_ShowTrayIcon( h_appExe, h_appExeIcon )
Return

h_SetHotkeyHelper:
   Suspend, On
   newKey =

   Gui, 2:+ToolWindow +AlwaysOnTop
   
   Gui, 2:Add, Text, Section, %lng_SetupUnhideKey%:
   Gui, 2:Add, Hotkey, ys w180 vnewKey Limit1, %h_hkKey%
   Gui, 2:Add, Text, xs w250, %lng_SetupHint% 
   Gui, 2:Font, bold
   Gui, 2:Add, Button, ym Section w90 gh_2ButtonOK Default, %lng_SetupOK%
   Gui, 2:Font
   Gui, 2:Add, Button, xs wp gh_2ButtonCancel, %lng_SetupCancel%
   
   Gui, 2:Show, Center, %h_MenuWin%
Return

h_2ButtonOK:
   Gui, 2:+OwnDialogs
   Gui, 2:Submit, NoHide

   If ( newKey = "" )
   {
      RegDelete, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|ch
      f_AssignHotkey( h_hkKey ) ; removes hotkey
      h_hkKey =
   }
   Else
   {
      ; trying to assign new hotkey
      success := f_AssignHotkey( newKey, "h_Unhide" )
      If ( success )
      {
         h_hkKey = %newKey%
         RegWrite, REG_SZ, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|ch, %h_hkKey%
      }
      Else
      {
         Msgbox, 48, %h_MenuWin%, %lng_SetupInvalidHotkey1%%lng_SetupUnhideKey%%lng_SetupInvalidHotkey2%
         Return
      }
   }
      
   GUI, 2:Destroy
   Suspend, Off
Return

h_2ButtonCancel:
2GuiEscape:
2GuiClose:
  newKey =

  GUI, 2:Destroy
  Suspend, Off
Return

h_BossKeyListToggle:
	; remove from/add to bosskey list and change tray menu accordingly
	If ( h_bkEnabled )
	{
		h_bkEnabled := FALSE
		RegDelete, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|bk
		Menu, TRAY, Rename, %lng_MenuOnBKList%, %lng_MenuNotOnBKList%


	}
	Else
	{
		h_bkEnabled := TRUE
		RegWrite, REG_SZ, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|bk, %h_bkEnabled%
		Menu, TRAY, Rename, %lng_MenuNotOnBKList%, %lng_MenuOnBKList%
	}
Return

;----------------------------------------------------------------------------
; Helper error -- subs
;

h_ExitWithParamError:
 Msgbox, 48, %lng_WindowTitle%, %lng_ExitWithParamError%
ExitApp, 1

h_ExitWithClassError:
 Msgbox, 48, %lng_WindowTitle%, %lng_ExitWithClassError%
ExitApp, 1

;=============================================================================
; Starter -- subs
;

h_InitStarter:
   ; lockfile checking
   m2t_lockFileName = %TEMP%\Min2Tray.lck
   
   IniRead, m2t_chkName, %m2t_lockFileName%, M2Tlock, Name
   IniRead, m2t_chkPID, %m2t_lockFileName%, M2Tlock, PID
   IniRead, m2t_chkDate, %m2t_lockFileName%, M2Tlock, Date

   If ( ( m2t_chkName <> "ERROR" ) And ( m2t_chkPID <> "ERROR" ) And ( m2t_chkDate <> "ERROR" ) )
   {
      Process, Exist, %m2t_chkPID%			; PID still existing...
      If ( ErrorLevel )
      {
      	Process, Exist, %m2t_chkName%		; and name still existing
      	If ( ErrorLevel )
      		Goto, ExitWithRunningError	; error&exit
      }
   }

   ; find AutoHotkey.exe and save its path to a variable for later use
   If ( Not A_IsCompiled )
   {
      RegRead, h_ahkFullPath, HKLM, SOFTWARE\AutoHotkey, InstallDir
      h_ahkFullPath = %h_ahkFullPath%\AutoHotkey.exe
      IfNotExist, %h_ahkFullPath%
      {
      	h_ahkFullPath := f_FindFile( "AutoHotkey.exe", A_ProgramFiles )
      	If ( h_ahkFullPath = "" )
      	{
      		StringSplit, tmparray, A_WinDir, \	; search whole sys-drive
      		h_ahkFullPath := f_FindFile( "AutoHotkey.exe", tmparray1 )
      		If ( h_ahkFullPath = "" )
      			Goto, ExitWithAHKError
      	}
      }
   }

   h_winPIDs :=  ; h_winPIDs%WIN_ID% -- array for WIN_IDs and corresponding PIDs
   h_helperPIDs :=	; h_helperPIDs%h_hPIDcount% -- array for PIDs of started helpers
   h_hPIDcount  := 0  

   Menu, TRAY, NoStandard
   Menu, TRAY, Add, %lng_MenuAbout%, h_AboutStarter
   Menu, TRAY, Add
   Menu, TRAY, Add, %lng_MenuBKPurgeList%, h_BossKeyPurgeList
   Menu, TRAY, Add, %lng_MenuSetHotkeysStarter%, h_SetHotkeyStarter
   Menu, TRAY, Add
   Menu, TRAY, Add, %lng_MenuRestoreOnly%, h_RestoreStarter
   Menu, TRAY, Add, %lng_MenuQuitRestoreAll%, h_QuitRestoreStarter
   Menu, TRAY, Add, %lng_MenuQuitOnly%, h_QuitStarter
   Menu, TRAY, Tip, %lng_TrayTitle%
   f_ShowTrayIcon( h_StarterIcon, h_StarterIcon# )

   ; assigning keys
   ; key for minimizing
   m2t_hkKey =
   RegRead, m2t_hkKey, HKCU, %h_RegSubkey%\Starter, HotkeyMinimize
   f_AssignHotkey( m2t_hkKey, "h_StartNewHelper" )
   ; stealthmode key
   m2t_hkStealth =
	RegRead, m2t_hkStealth, HKCU, %h_RegSubkey%\Starter, HotkeyStealth
	If ( ErrorLevel OR m2t_hkStealth = "" )
	   m2t_hkStealth = ^#!PgUp   ; default: Ctrl+Win+Alt+PgUp
	f_AssignHotkey( m2t_hkStealth, "h_StealthModeToggle" )
	; get bosskey mode
	RegRead, bkOptIn, HKCU, %h_RegSubkey%\Misc, BossKeyOptIn
	If ( ErrorLevel OR Not bkOptIn )
	   bkOptIn := FALSE	; default: Opt-Out mode
	Else
	   bkOptIn := TRUE
   ; bosskey
   m2t_hkBossKey =
	RegRead, m2t_hkBossKey, HKCU, %h_RegSubkey%\Starter, HotkeyBossKey
	f_AssignHotkey( m2t_hkBossKey, "h_BossKey" )
   ; middle mouse button
   RegRead, m2t_hkMButton, HKCU, %h_RegSubkey%\Starter, HotkeyMButton
   If ( m2t_hkMButton = 1 OR m2t_hkMButton = 2 )
      f_AssignHotkey( "MButton", "h_StartNewHelper" )
   Else ; default: set mbutton-action off
      m2t_hkMButton := 0
   
   ; create temporary lockfile
   ; info: WinGet can only retrieve the ProcessName after setting up tray
   Process, Exist, 
   m2t_ownPID = %ErrorLevel%
   WinGet, m2t_ownName, ProcessName, ahk_pid %m2t_ownPID%
   
   IniWrite, %m2t_ownName%, %m2t_lockFileName%, M2Tlock, Name
   IniWrite, %m2t_ownPID%, %m2t_lockFileName%, M2Tlock, PID
   IniWrite, %A_Now%, %m2t_lockFileName%, M2Tlock, Date
Return

h_StartNewHelper:
   If ( A_ThisHotkey = m2t_hkKey OR m2t_hkMButton = 2 ) ; what key triggered the action?
      h_fnStartNewHelper( TRUE )  ; no check for title-click
   Else
      h_fnStartNewHelper()
Return

h_fnStartNewHelper( h_noCheck=FALSE, h_WinID=FALSE )
{
	Global  ; assumption: all vars are global except that are declared Local
	Local h_CaptionHeight, h_BorderHeight, h_MouseX, h_MouseY, h_WindowWidth, h_WinStyle, h_ID, h_newPID

	If ( h_WinID )
	{
		IfWinNotExist, ahk_id %h_WinID%	; set "last used window"
			Return
	}
	Else
	{
		WinWait, A,, 2			; set "last used window"
		If ( ErrorLevel )	; It timed out, so do nothing
			Return
	}

	If ( h_noCheck )
		Goto, h_suStartNewHelperMinimize  ; minimize immedieatly
	Else ; otherwise check if middle-click on title-bar
	{
    CoordMode, Mouse, Relative
    SysGet, h_CaptionHeight, 4	; SM_CYCAPTION
    SysGet, h_BorderHeight, 7	; SM_CXDLGFRAME
    MouseGetPos, h_MouseX, h_MouseY

    If ( ( h_MouseY > 0 ) And ( h_MouseY < h_CaptionHeight + h_BorderHeight ) )
    {
      WinGetPos, , , h_WindowWidth
      If ( ( h_MouseX > 0 ) And ( h_MouseX < h_WindowWidth ) )
        Goto, h_suStartNewHelperMinimize
    }
    Else
      MouseClick, Middle
  }
  Return

  ; sub-routine in function
  h_suStartNewHelperMinimize:
    WinGet, h_WinStyle, Style
    
    ; checks for ( WS_VISIBLE | WS_THICKFRAME | WS_MINIMIZEBOX )
    If ( h_WinStyle & 0x10060000 = 0x10060000 )
    {
    	WinGet, h_ID, ID

      h_process := h_winPIDs%h_ID%  ; get PID of helper for WIN_ID if already started
      If ( h_process <> "" )
      {
        Process, Exist, %h_process%
        If ( ErrorLevel )	; helper still exists
          Return          ; -> do not minimize window again
      }
      ; -> otherwise: start new helper
      h_winPIDs%h_ID% :=  ; clear var with new WIN_ID

    	If ( A_IsCompiled )
    		Run, %A_ScriptFullPath% %h_ID%, , UseErrorLevel, h_newPID
    	Else
    		Run, %h_ahkFullPath% %A_ScriptFullPath% %h_ID%, , UseErrorLevel, h_newPID
    
    	If ( ErrorLevel = "ERROR" )	; helper could not be started
    		Return
    
    	h_winPIDs%h_ID% = %h_newPID% ; assign new PID
    	h_hPIDcount++
    	h_helperPIDs%h_hPIDcount% = %h_newPID%
    }
  Return
}

h_AboutStarter:
 Msgbox, 64, %lng_WindowTitle%, %lng_About%
Return

h_RestoreStarter:	; close all helpers -> unhide every window
 Loop
 {
	If ( h_hPIDcount = 0 )
		Break
	h_process := h_helperPIDs%h_hPIDcount%
	Process, Exist, %h_process%
	If ( ErrorLevel )	; helper still exists
	{
		WinClose, ahk_pid %h_process%
	}
	h_hPIDcount--
	Sleep, 50
 }
Return

h_QuitRestoreStarter:
 Gosub, h_RestoreStarter
 ; and quit
 ; ...
h_QuitStarter:
 Gosub, h_CleanUpLockFile
ExitApp

h_CleanUpLockFile:
 IfNotExist, %m2t_lockFileName%
	Return

 FileDelete, %m2t_lockFileName%
 If ( ErrorLevel )
 {
	FileSetAttrib, -RHS, %m2t_lockFileName%	; one more try...
	FileDelete, %m2t_lockFileName%
	If ( ErrorLevel )
		ExitApp, 2		; fuck it! and exit with code "2"
 }
Return

h_SetHotkeyStarter:
   Suspend, On
   newKey =
   newBossKey =
   
   Gui, +ToolWindow +AlwaysOnTop
   ; hotkey + mbutton
   Gui, Add, Text, Section, %lng_SetupKey%:
   Gui, Add, Hotkey, ys w180 vnewKey Limit1, %m2t_hkKey%
   Gui, Add, Text, xs w250 R3, %lng_SetupHint% 
   tmp0 = 0
   tmp1 = 0
   tmp2 = 0
   tmp%m2t_hkMButton% = 1 ; check the active radiobox
   Gui, Add, Text, xs w250, %lng_SetupMButtonTitle%
   Gui, Add, Radio, xs w230 vnewMButton0 Checked%tmp0%, %lng_SetupMButtonOpt0%
   Gui, Add, Radio, xs w230 vnewMButton1 Checked%tmp1%, %lng_SetupMButtonOpt1%
   Gui, Add, Radio, xs w230 vnewMButton2 Checked%tmp2%, %lng_SetupMButtonOpt2%
   ; bosskey
   Gui, Add, Text, ym Section, %lng_SetupBossKey%:
   Gui, Add, Hotkey, ys w180 vnewBossKey Limit1, %m2t_hkBossKey%
   Gui, Add, Text, xs w250 R3, %lng_SetupHint% 
   tmp0 = 0
   tmp1 = 0
   tmp%bkOptIn% = 1 ; check the active radiobox
   Gui, Add, Text, xs w250, %lng_SetupBKTitle%
   Gui, Add, Radio, xs w230 vnewBKmode0 Checked%tmp0%, %lng_SetupBKOpt0%
   Gui, Add, Radio, xs w230 vnewBKmode1 Checked%tmp1%, %lng_SetupBKOpt1%
   ; ok + cancel
   Gui, Font, bold
   Gui, Add, Button, ym Section w90 gh_ButtonOK Default, %lng_SetupOK%
   Gui, Font
   Gui, Add, Button, xs wp gh_ButtonCancel, %lng_SetupCancel%
   
   Gui, Show, Center, %lng_SetupTitle%
Return

h_ButtonOK:
   Gui, +OwnDialogs
   Gui, Submit, NoHide

   ; hotkey for minimizing
   If ( newKey = "" )
   {
      RegDelete, HKCU, %h_RegSubkey%\Starter, HotkeyMinimize
      f_AssignHotkey( m2t_hkKey ) ; remove old hotkey
      m2t_hkKey =
   }
   Else
   {
      ; trying to assign new hotkey
      success := f_AssignHotkey( newKey, "h_StartNewHelper" )
      If ( success )
      {
         m2t_hkKey = %newKey%
         RegWrite, REG_SZ, HKCU, %h_RegSubkey%\Starter, HotkeyMinimize, %m2t_hkKey%
      }
      Else
      {
         Msgbox, 48, %h_MenuWin%, %lng_SetupInvalidHotkey1%%lng_SetupKey%%lng_SetupInvalidHotkey2%
         Return
      }
   }
   ; bosskey  
   If ( newBossKey = "" )
   {
      RegDelete, HKCU, %h_RegSubkey%\Starter, HotkeyBossKey
      f_AssignHotkey( m2t_hkBossKey ) ; remove old hotkey
      m2t_hkBossKey =
   }
   Else
   {
      ; trying to assign new hotkey
      success := f_AssignHotkey( newBossKey, "h_BossKey" )
      If ( success )
      {
         m2t_hkBossKey = %newBossKey%
         RegWrite, REG_SZ, HKCU, %h_RegSubkey%\Starter, HotkeyBossKey, %m2t_hkBossKey%
      }
      Else
      {
         Msgbox, 48, %h_MenuWin%, %lng_SetupInvalidHotkey1%%lng_SetupBossKey%%lng_SetupInvalidHotkey2%
         Return
      }
   }
   ; bosskey mode
   If ( newBKmode1 ) ; opt-in
      bkOptIn := TRUE
   Else ; default: opt-out
      bkOptIn := FALSE 
   RegWrite, REG_SZ, HKCU, %h_RegSubkey%\Misc, BossKeyOptIn, %bkOptIn%
   ; mbutton mode
   If ( newMButton1 ) ; title-click
      m2t_hkMButton := 1
   Else If ( newMButton2 ) ; just third mouse button
      m2t_hkMButton := 2
   Else ; default: set mbutton-action off
      m2t_hkMButton := 0 
   If ( m2t_hkMButton = 1 OR m2t_hkMButton = 2 )
      f_AssignHotkey( "MButton", "h_StartNewHelper" )
   Else ; default: set mbutton-action off
      f_AssignHotkey( "MButton" )
   RegWrite, REG_SZ, HKCU, %h_RegSubkey%\Starter, HotkeyMButton, %m2t_hkMButton%

   GUI, Destroy
   Suspend, Off
Return

h_ButtonCancel:
GuiEscape:
GuiClose:
  newKey =
  newMButton =

  GUI, Destroy
  Suspend, Off
Return

h_BossKey:
	; build list with currently active (visible) windows
	DetectHiddenWindows, Off
	bkWinID := 0
	WinGet, bkWinID, List
	DetectHiddenWindows, On
	; go thru list and hide all windows, except ones that are opt'ed-out by user
	Loop, %bkWinID%
	{
		tmpWinID := bkWinID%A_Index%
		WinGetClass, bkClass, ahk_id %tmpWinID%
		WinGet, bkApplication, ProcessName, ahk_id %tmpWinID%
		RegRead, bkEnabled, HKCU, %h_RegSubkey%, %bkApplication%|%bkClass%|bk	; app+class
		If ( Not bkEnabled )
			RegRead, bkEnabled, HKCU, %h_RegSubkey%, %bkApplication%|bk 	; generic app (without class)
		; opt-out mode: hide all windows, BUT user-specified ones
		; opt-in mode : hide ONLY user-specified windows
		; hide if ((optin AND window) OR (optout AND not-window))
		If ( ( bkOptIn AND bkEnabled ) OR ( Not bkOptIn AND Not bkEnabled ) )
			h_fnStartNewHelper( TRUE, tmpWinID )	; unconditional hide of window
	}
	; make first remaining visible window active again, if any
	Sleep, 50
	KeyWait, %m2t_hkBossKey%, L  ; wait for the bosskey to be logically released
	DetectHiddenWindows, Off
	bkWinID := 0
	WinGet, bkWinID, List
	DetectHiddenWindows, On
	Loop, %bkWinID%
	{
		tmpWinID := bkWinID%A_Index%
		WinGetClass, bkClass, ahk_id %tmpWinID%
		If bkClass not in Shell_TrayWnd,Progman
		{
			WinActivate, ahk_id %tmpWinID%
			Break
		}
	}
Return

h_BossKeyPurgeList:
	Msgbox, 36, %lng_WindowTitle%, %lng_BKPurgeList%
	IfMsgBox, No	; do not remove
		Return

	Loop, HKCU, %h_RegSubkey%
	{
		StringRight, tmp, A_LoopRegName, 3
		If ( tmp = "|bk" )
			RegDelete
	}
Return

h_StealthModeToggle:
	RegRead, tmp, HKCU, %h_RegSubkey%\Misc, StealthMode
	tmp := Not tmp
	RegWrite, REG_SZ, HKCU, %h_RegSubkey%\Misc, StealthMode, %tmp%
	; update/remove icon for Starter
	f_ShowTrayIcon( h_StarterIcon, h_StarterIcon# )
Return

;=============================================================================
; Starter error -- subs
;

ExitWithAHKError:
 Msgbox, 48, %lng_WindowTitle%, %lng_ExitWithAHKError%
ExitApp, 1

ExitWithRunningError:
 Msgbox, 48, %lng_WindowTitle%, %lng_ExitWithRunningError%
ExitApp, 1

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; general -- subs
;

h_InitGeneral:
   ; key in HKCU to store stuff under
   h_RegSubkey = Software\KTC\Min2Tray
   
   ; l10n
   StringRight, tmp, A_Language, 2
   If ( tmp = 07 )	; = Deutsch (0407, 0807, 0c07 ...)
   {
      FormatTime, tmpDate, %versionDate%, dd.MM.yyyy
      ; General
      lng_WindowTitle            = Min2Tray
      lng_SetupOK                = bernehmen
      lng_SetupCancel            = Abbrechen
      lng_SetupHint              = Hinweis: Drcken Sie ENTFERNEN und besttigen Sie mit "%lng_SetupOK%", um den Hotkey zu lschen!
      lng_SetupInvalidHotkey1    = Folgender Hotkey konnte nicht zugewiesen werden: "
      lng_SetupInvalidHotkey2    = ".`nBitte anderen Hotkey whlen oder den Hotkey entfernen!
      ; Helper
      lng_MenuChangeName         = &Eigener Name...
      lng_MenuRemoveName         = &Entferne eigenen Name
      lng_MenuChangeIcon         = &Eigenes Icon...
      lng_MenuRemoveIcon         = &Entferne eigenes Icon
      lng_MenuAssignHotkey       = &Hotkey zuweisen oder lschen...
      lng_MenuClose              = &Schliee Fenster
      lng_MenuUnhide             = &Zeige Fenster
      lng_hClose1                = Wollen Sie das Fenster "
      lng_hClose2                = " wirklich schlieen?
      lng_ChangeNameSelector     = Geben Sie bitte den neuen Namen ein:
      lng_ChangeIconSelector     = Whlen Sie eine Icon-Datei aus
      lng_RemoveName             = Wollen Sie den benutzerdefinierten Name wirklich entfernen?
      lng_RemoveIcon             = Wollen Sie das benutzerdefinierte Icon wirklich entfernen?
      lng_MenuOnBKList           = &Entferne Fenster aus BossKey-Liste
      lng_MenuNotOnBKList        = &Fge Fenster zur BossKey-Liste hinzu
      lng_SetupUnhideKey         = Fenster zeigen
      ; Starter
      lng_MenuAbout              = &ber %lng_WindowTitle%
      lng_MenuRestoreOnly        = &Alle Fenster wiederherstellen
      lng_MenuQuitRestoreAll     = &Wiederherstellen && beenden
      lng_MenuQuitOnly           = Nur &Beenden
      lng_MenuSetHotkeysStarter  = &Einstellungen...
      lng_MenuBKPurgeList        = &BossKey-Liste leeren
      lng_BKPurgeList            = Wollen Sie die BossKey-Liste leeren?
      lng_TrayTitle              = %lng_WindowTitle% - Haupt
      lng_About                  = %lng_WindowTitle% v%versionString% (%tmpDate%)`n`nMinimiert Fenster in den Tray-Bereich als kleine Icons.`nDazu den dritten Mausknopf bzw. den benutzerdefinierten Hotkey drcken.`n`nVertrieben unter den Auflagen der GPLv2 oder neuer.`n 2005-2006 bei Junyx / KTC^brain`n
      lng_SetupTitle             = %lng_WindowTitle% Einstellungen
      lng_SetupKey               = Minimiere Fenster
      lng_SetupMButtonTitle      = Dritte (mittlere) Maustaste zum Minimieren nutzen?
      lng_SetupMButtonOpt0       = Kein Bedarf!
      lng_SetupMButtonOpt1       = durch Klicken auf Titelleiste
      lng_SetupMButtonOpt2       = durch Klicken (egal wohin)
      lng_SetupBossKey           = BossKey
      lng_SetupBKTitle           = BossKey-Modus bestimmen
      lng_SetupBKOpt0            = Opt-Out  -  minimiere alle Fenster AUSSER die der BossKey-Liste
      lng_SetupBKOpt1            = Opt-In  -  minimiere NUR Fenster der BossKey-Liste
      ; Error
      lng_ExitWithOSError        = Fehler: Falsches Betriebssystem. Bentige Windows NT und hher!
      lng_ExitWithVersionError   = Fehler: Bentige "AutoHotkey.exe" Version 1.0.42.02 oder hher!
      lng_ExitWithParamError     = Fehler: Keine Nummer als erster Parameter bergeben!
      lng_ExitWithClassError     = Fehler: Der Desktop und das Tray-Men knnen nicht versteckt werden!
      lng_ExitWithAHKError       = Fehler: Konnte Datei "AutoHotkey.exe" nicht ausfindig machen!
      lng_ExitWithRunningError   = Fehler: Min2Tray wurde schon einmal gestartet!
   }
   Else
   {
      FormatTime, tmpDate, %versionDate%, ShortDate
      ; General
      lng_WindowTitle            = Min2Tray
      lng_SetupOK                = &Apply
      lng_SetupCancel            = &Cancel
      lng_SetupHint              = Hint: Press DELETE on keyboard and confirm by clicking "%lng_SetupOK%" to clear the hotkey!
      lng_SetupInvalidHotkey1    = The following hotkey could not be assigned: "
      lng_SetupInvalidHotkey2    = ".`nPlease use a different hotkey or delete the hotkey!
      ; Helper
      lng_MenuChangeName         = &Custom name...
      lng_MenuRemoveName         = &Restore original name
      lng_MenuChangeIcon         = &Custom icon...
      lng_MenuRemoveIcon         = &Remove custom icon
      lng_MenuAssignHotkey       = &Assign or clear hotkey...
      lng_MenuClose              = &Close window
      lng_MenuUnhide             = &Show window
      lng_hClose1                = Do you really wanna close window "
      lng_hClose2                = "?
      lng_ChangeNameSelector     = Please enter the custom name:
      lng_ChangeIconSelector     = Select a Icon file
      lng_RemoveName             = Do you really wanna remove the custom name?
      lng_RemoveIcon             = Do you really wanna remove the custom icon?
      lng_MenuOnBKList           = &Remove window from BossKey list
      lng_MenuNotOnBKList        = &Add window to BossKey list
      lng_SetupUnhideKey         = Show window
      ; Starter
      lng_MenuAbout              = &About %lng_WindowTitle%
      lng_MenuRestoreOnly        = &Restore all windows
      lng_MenuQuitRestoreAll     = Restore &all && quit
      lng_MenuQuitOnly           = &Quit only
      lng_MenuSetHotkeysStarter  = &Preferences...
      lng_MenuBKPurgeList        = Purge &BossKey list
      lng_BKPurgeList            = Do you really wanna purge the BossKey list?
      lng_TrayTitle              = %lng_WindowTitle% - Main
      lng_About                  = %lng_WindowTitle% v%versionString% (%tmpDate%)`n`nMinimize windows to the Tray area as icons.`nPress third mouse button or custom hotkey.`n`nDistributed under the terms of GPLv2 or newer.`n 2005-2006 by Junyx / KTC^brain`n
      lng_SetupTitle             = %lng_WindowTitle% Prefs
      lng_SetupKey               = Minimize window
      lng_SetupMButtonTitle      = Use third (middle) mouse button for minimizing?
      lng_SetupMButtonOpt0       = Nope!
      lng_SetupMButtonOpt1       = by clicking on titlebar
      lng_SetupMButtonOpt2       = by clicking (no matter where)
      lng_SetupBossKey           = BossKey
      lng_SetupBKTitle           = Set BossKey mode
      lng_SetupBKOpt0            = Opt-Out  -  minimize all windows EXCEPT those from BossKey list
      lng_SetupBKOpt1            = Opt-In  -  minimize ONLY windows from BossKey list
      ; Error
      lng_ExitWithOSError        = Error: Wrong operating system. Need Windows NT or newer!
      lng_ExitWithVersionError   = Error: Need AutoHotkey.exe version 1.0.42.02 or newer!
      lng_ExitWithParamError     = Error: First parameter must be a number!
      lng_ExitWithClassError     = Error: Desktop and Tray Menu can not be hidden!
      lng_ExitWithAHKError       = Error: Could not find file "AutoHotkey.exe"!
      lng_ExitWithRunningError   = Error: Min2Tray is already running!
   }

   ; some sanity checks   
   If ( A_OSType <> "WIN32_NT" )
   {
      Msgbox, 48, %lng_WindowTitle%, %lng_ExitWithOSError%
      ExitApp, 1
   }
   If ( A_AhkVersion < "1.0.42.02" )
   {
      Msgbox, 48, %lng_WindowTitle%, %lng_ExitWithVersionError%
      ExitApp, 1
   }

   ; tray icon for starter and as default
   If ( A_IsCompiled )
   {
      h_StarterIcon  = %A_ScriptFullPath%
      h_StarterIcon# = 1
   }
   Else
   {
      h_StarterIcon  = %A_WinDir%\system32\shell32.dll
      h_StarterIcon# = 25
   }
Return

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; general -- functions
;

f_FindFile( name, startPath )
{
   StringRight, tmp, startPath, 1
   If ( ( tmp <> "" ) and ( tmp <> "\" ) )
      startPath = %startPath%\
   ; start file search
   Loop, %startPath%%name%, 0, 1
   {
      appExe = %A_LoopFileLongPath%
      IfExist, %appExe%
         Break		; exit with first found app or ""
   }
   Return %appExe%
}

f_ShowTrayIcon( file, number )
{
   Global h_RegSubkey, h_StarterIcon, h_StarterIcon#

   Menu, TRAY, UseErrorLevel
	RegRead, stealth, HKCU, %h_RegSubkey%\Misc, StealthMode	; are we in StealthMode?
	If ( Not ErrorLevel )
		If ( stealth )
		{
			If ( Not A_IconHidden )
				Menu, TRAY, NoIcon
			Return
		}

	If ( Not A_IconHidden )	; coming from hidden icon: ALWAYS renew icon!
		If ( A_IconFile = file AND A_IconNumber = number )	; icon props not changed
			Return

   IfExist, %file%
   {
      Menu, TRAY, Icon, %file%, %number%, 1  ; 1=freeze icon
      If ( Not ErrorLevel )	; if ErrorLevel was NOT raised -> allright
         Goto, OKout
   }

   ; if ErrorLevel or not existing file -> set default icon
   Menu, TRAY, Icon, %h_StarterIcon%, %h_StarterIcon#%, 1 ; 1=freeze icon

   OKout:
      Menu, TRAY, Icon	; show icon
      Menu, TRAY, UseErrorLevel, Off
   Return
}

f_AssignHotkey( hkey, hlabel="" )
{  ; - assign exactly one hotkey to a label
   ; - return TRUE if assignment succeeded
   ;   otherwise FALSE
   ; - remove hotkey if no label is passed
   ;   (return value is not relevant here)
   retVal := FALSE
   ; disable any previosly assigned hotkey
   Hotkey, %hkey%, Off, UseErrorLevel
   ; if no label is specified -> only remove given hotkey
   If IsLabel( hlabel )
   {  ; assign new key
      Hotkey, %hkey%, %hlabel%, UseErrorLevel
      If ( Not ErrorLevel )
         Hotkey, %hkey%, On
            If ( Not ErrorLevel )
               retVal := TRUE
   }
   Return retVal
}

;EOF

